/* Copyright 2002 Red Hat, Inc.  All Rights Reserved.  Distribute under 
 * the terms of the GPLv2 or later.
 */
#define _XOPEN_SOURCE		500	/* for pread/pwrite */
#define _FILE_OFFSET_BITS	64

#include <sys/time.h>
#include <sys/resource.h>
#include <sys/fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

/* 10.t - uses testdir.enospc/rwfile
- Check results on out-of-space and out-of-quota. (10.t)
        - write that fills filesystem but does not go over should succeed
        - write that fills filesystem and goes over should be partial
        - write to full filesystem should return -ENOSPC
        - read beyond end of file after ENOSPC should return 0
*/
#define READ		'r'
#define WRITE		'w'
#define READ_SILENT	'R'
#define WRITE_SILENT	'W'
int attempt_rw(int fd, void *buf, int count, long long pos, int rw, int expect)
{
	int res;
	int silent = 0, do_read = 1;

	switch(rw) {
	case READ_SILENT:
		silent = 1;
	case READ:
		break;
	case WRITE_SILENT:
		silent = 1;
	case WRITE:
		do_read = 0;
		break;
	}

	if (!silent) {
		printf("expect %5d: (%c), res = ", expect, rw);
		fflush(stdout);
	}
	if (do_read)
		res = pread(fd, buf, count, pos);
	else
		res = pwrite(fd, buf, count, pos);

	if (!silent || res != expect) {
		if (silent)
			printf("expect %5d: (%c), res = ", expect, rw);
		printf("%5d [%s]%s\n", res,
			(res <= 0) ? strerror(-res) : "Success",
			(res != expect) ? " -- FAILED" : "");
	}

	if (res != expect)
		return 1;

	return 0;
}

int test_main(void)
{
#define LIMIT	8192
#define SIZE	8192
	char *buf;
	int rwfd;
	int status = 0, res;

	rwfd = open("testdir.enospc/rwfile", O_RDWR|O_CREAT|O_TRUNC, 0600);
							assert(rwfd != -1);
	res = ftruncate(rwfd, 0);			assert(res == 0);
	buf = malloc(SIZE);				assert(buf != NULL);
	memset(buf, 0, SIZE);


	status |= attempt_rw(rwfd, buf, SIZE,   LIMIT-SIZE, WRITE, SIZE);
	status |= attempt_rw(rwfd, buf, SIZE,   LIMIT-SIZE,  READ, SIZE);

	status |= attempt_rw(rwfd, buf, SIZE,        LIMIT, WRITE, -ENOSPC);
	status |= attempt_rw(rwfd, buf, SIZE,        LIMIT,  READ,       0);

	res = ftruncate(rwfd, 0);			assert(res == 0);

	status |= attempt_rw(rwfd, buf, SIZE, 1+LIMIT-SIZE, WRITE, SIZE-1);
	status |= attempt_rw(rwfd, buf, SIZE, 1+LIMIT-SIZE,  READ, SIZE-1);
	status |= attempt_rw(rwfd, buf, SIZE,        LIMIT,  READ,      0);

	status |= attempt_rw(rwfd, buf, SIZE,        LIMIT, WRITE, -ENOSPC);
	status |= attempt_rw(rwfd, buf, SIZE,        LIMIT,  READ,       0);
	status |= attempt_rw(rwfd, buf,    0,        LIMIT, WRITE,       0);

	res = close(rwfd);				assert(res == 0);
	res = unlink("testdir.enospc/rwfile");		assert(res == 0);
	return status;
}

int main(void)
{
        int res;

        res = test_main();
        printf("test %s completed %s.\n", "foo", 
                res ? "FAILED" : "PASSED"
                );
        fflush(stdout);
        return res ? 1 : 0;
}

